import c4d
from c4d import plugins

import mxutils


class Audioworkstation(plugins.TagData):
    """Your plugin class.
    """
    PLUGIN_ID: int = 7654321
 
    def Execute(self, tag, doc, op, bt, priority, flags): return c4d.EXECUTIONRESULT_OK

    def __init__(self) -> None:
        """
        """
        self.buttonIds: list[int] = [2000]

    def Init(self, node, isCloneInit) -> bool:
        """Init our parameters.
        """
        self.InitAttr(node, int, c4d.ID_AUDIOWORKSTATION_ITEMS)
        if not isCloneInit:
            node[c4d.ID_AUDIOWORKSTATION_ITEMS] = 0

        return True

    def GetDDescription(self, op: c4d.BaseTag, description: c4d.Description, 
                        flags: int) -> tuple[bool, int]:
        """More or less your code, I just streamlined things a bit.
        """
        if not description.LoadDescription(Audioworkstation.PLUGIN_ID):
            return False
        
        target: c4d.DescID | None = description.GetSingleDescID()
        buttonIds: list[c4d.DescID] = [
            c4d.DescID(c4d.DescLevel(did, c4d.DTYPE_BUTTON, Audioworkstation.PLUGIN_ID)) 
            for did in self.buttonIds
        ]

        for eid in buttonIds:
            # The eid.IsPartOf(target) is an echo of what you did in your code which does not make too 
            # much sense. I copied it here as it also does not hurt. eid.IsPartOf(target) basically
            # says "if the made up id #eid of the to be added element is part of the description
            # element Cinema 4D is currently polling the description for, then do this". Which does
            # not make too much sense. It is the #target or the #singleDescId condition in your code 
            # which makes this work.
            if target and not eid.IsPartOf(target)[0]:
                continue

            buttonDesc: c4d.BaseContainer = c4d.GetCustomDataTypeDefault(c4d.DTYPE_BUTTON)
            buttonDesc[c4d.DESC_NAME] = str(eid)
            buttonDesc[c4d.DESC_CUSTOMGUI] = c4d.CUSTOMGUI_BUTTON
            if not description.SetParameter(eid, buttonDesc, c4d.DescID(c4d.ID_TAGPROPERTIES)):
                return False

        return True, flags | c4d.DESCFLAGS_DESC_LOADED

    def Message(self, node: c4d.BaseTag, mtype: int, mdata: any) -> bool:
        """
        """
        # Our Add button is being pressed, we want to add an item to the UI.
        if (mtype == c4d.MSG_DESCRIPTION_COMMAND and isinstance(mdata, dict)):
            descId: c4d.DescID = mxutils.CheckType(mdata.get("id", None))
            if descId.GetDepth() < 1 or descId[0].id != c4d.ID_AUDIOWORKSTATION_ADD:
                return True
            
            doc: c4d.documents.BaseDocument = node.GetDocument()
            if not doc:
                return True
            
            # We add a new ID to our button IDs.
            self.buttonIds.append(self.buttonIds[-1] + 100)

            # Now we have two options, either we do what the constraint tag does, and now modify
            # the data container of the node as a result of the button press and with that trigger
            # the MSG_DESCRIPTION_POSTSETPARAMETER message.
            
            # doc.StartUndo()
            # doc.AddUndo(c4d.UNDOTYPE_CHANGE, node)
            # node[c4d.ID_AUDIOWORKSTATION_ITEMS] = node[c4d.ID_AUDIOWORKSTATION_ITEMS] + 1
            # doc.EndUndo()

            # Or we just flag ourselves as data dirty, i.e., we pretend that our data has changed,
            # this is enough to sweet talk Cinema into a description update.
            node.SetDirty(c4d.DIRTYFLAGS_DATA)

            # PS: I also tried c4d.CoreMessage(c4d.COREMSG_CINEMA_FORCE_AM_UPDATE) but that does not
            # work here.
            
            return True
        return True

# main
if __name__ == "__main__":    
    c4d.plugins.RegisterTagPlugin(id=Audioworkstation.PLUGIN_ID,
                                  str="C4D-Audioworkstation",
                                  info=c4d.TAG_EXPRESSION | c4d.TAG_VISIBLE,
                                  description="audioworkstation",
                                  g=Audioworkstation, icon=None)

